package kr.co.vividnext.sodalive.admin.content.theme

import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.theme.AudioContentTheme
import kr.co.vividnext.sodalive.content.theme.GetAudioContentThemeResponse
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile

@Service
class AdminContentThemeService(
    private val s3Uploader: S3Uploader,
    private val objectMapper: ObjectMapper,
    private val repository: AdminContentThemeRepository,

    @Value("\${cloud.aws.s3.bucket}")
    private val bucket: String
) {
    @Transactional
    fun uploadThemeImage(image: MultipartFile, requestString: String) {
        val request = objectMapper.readValue(requestString, CreateContentThemeRequest::class.java)
        themeExistCheck(request)

        val fileName = generateFileName()
        val imagePath = s3Uploader.upload(
            inputStream = image.inputStream,
            bucket = bucket,
            filePath = "audio_content_theme/$fileName"
        )

        return createTheme(request.theme, imagePath)
    }

    fun createTheme(theme: String, imagePath: String) {
        repository.save(AudioContentTheme(theme = theme, image = imagePath))
    }

    fun themeExistCheck(request: CreateContentThemeRequest) {
        repository.findIdByTheme(request.theme)?.let { throw SodaException("이미 등록된 테마 입니다.") }
    }

    @Transactional
    fun deleteTheme(id: Long) {
        val theme = repository.findByIdOrNull(id)
            ?: throw SodaException("잘못된 요청입니다.")

        theme.theme = "${theme.theme}_deleted"
        theme.isActive = false
    }

    @Transactional
    fun updateTagOrders(ids: List<Long>) {
        for (index in ids.indices) {
            val theme = repository.findByIdOrNull(ids[index])

            if (theme != null) {
                theme.orders = index + 1
            }
        }
    }

    fun getThemes(): List<GetAudioContentThemeResponse> {
        return repository.getActiveThemes()
    }
}
