package kr.co.vividnext.sodalive.admin.member.tag

import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.DeleteMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestPart
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@RequestMapping("/admin/member/tag")
@PreAuthorize("hasRole('ADMIN')")
class AdminMemberTagController(private val service: AdminMemberTagService) {
    @PostMapping
    fun enrollmentCreatorTag(
        @RequestPart("image") image: MultipartFile,
        @RequestPart("request") requestString: String
    ) = ApiResponse.ok(service.uploadTagImage(image, requestString), "등록되었습니다.")

    @DeleteMapping("/{id}")
    fun deleteCreatorTag(@PathVariable id: Long) = ApiResponse.ok(service.deleteTag(id), "삭제되었습니다.")

    @PutMapping("/{id}")
    fun modifyCreatorTag(
        @PathVariable id: Long,
        @RequestPart("image") image: MultipartFile?,
        @RequestPart("request") requestString: String
    ) = ApiResponse.ok(service.modifyTag(id, image, requestString), "수정되었습니다.")

    @PutMapping("/orders")
    fun updateTagOrders(
        @RequestBody request: UpdateTagOrdersRequest
    ) = ApiResponse.ok(service.updateTagOrders(request.ids), "수정되었습니다.")
}
