package kr.co.vividnext.sodalive.content.pin

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.pin.QPinContent.pinContent
import org.springframework.data.jpa.repository.JpaRepository

interface PinContentRepository : JpaRepository<PinContent, Long>, PinContentQueryRepository

interface PinContentQueryRepository {
    fun getPinContentList(memberId: Long): List<PinContent>

    fun findByContentIdAndMemberId(contentId: Long, memberId: Long, active: Boolean? = null): PinContent?
}

class PinContentQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : PinContentQueryRepository {
    override fun getPinContentList(memberId: Long): List<PinContent> {
        return queryFactory
            .selectFrom(pinContent)
            .where(pinContent.member.id.eq(memberId))
            .orderBy(pinContent.updatedAt.asc())
            .fetch()
    }

    override fun findByContentIdAndMemberId(contentId: Long, memberId: Long, active: Boolean?): PinContent? {
        var where = pinContent.content.id.eq(contentId)
            .and(pinContent.member.id.eq(memberId))

        if (active != null) {
            where = where
                .and(pinContent.isActive.eq(active))
        }

        return queryFactory
            .selectFrom(pinContent)
            .where(where)
            .fetchFirst()
    }
}
