package kr.co.vividnext.sodalive.content.theme

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.theme.QAudioContentTheme.audioContentTheme
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Repository

@Repository
class AudioContentThemeQueryRepository(
    private val queryFactory: JPAQueryFactory,
    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) {
    fun getActiveThemes(): List<GetAudioContentThemeResponse> {
        return queryFactory
            .select(
                QGetAudioContentThemeResponse(
                    audioContentTheme.id,
                    audioContentTheme.theme,
                    audioContentTheme.image.prepend("/").prepend(cloudFrontHost)
                )
            )
            .from(audioContentTheme)
            .where(audioContentTheme.isActive.isTrue)
            .orderBy(audioContentTheme.orders.asc())
            .fetch()
    }

    fun getActiveThemeOfContent(isAdult: Boolean = false): List<String> {
        var where = audioContent.isActive.isTrue
            .and(audioContentTheme.isActive.isTrue)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(audioContentTheme.theme)
            .from(audioContent)
            .innerJoin(audioContent.theme, audioContentTheme)
            .where(where)
            .groupBy(audioContentTheme.id)
            .orderBy(audioContentTheme.orders.asc())
            .fetch()
    }

    fun findThemeByIdAndActive(id: Long): AudioContentTheme? {
        return queryFactory
            .selectFrom(audioContentTheme)
            .where(
                audioContentTheme.id.eq(id)
                    .and(audioContentTheme.isActive.isTrue)
            )
            .fetchFirst()
    }
}
