package kr.co.vividnext.sodalive.event

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.event.QEvent.event
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface EventRepository : JpaRepository<Event, Long>, EventQueryRepository

interface EventQueryRepository {
    fun getEventList(isAdult: Boolean? = null): List<EventItem>
    fun getMainEventPopup(isAdult: Boolean): EventItem?
}

@Repository
class EventQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : EventQueryRepository {
    override fun getEventList(isAdult: Boolean?): List<EventItem> {
        var where = event.isActive.isTrue

        if (isAdult != null) {
            where = if (isAdult) {
                where.and(
                    event.isAdult.isNull
                        .or(event.isAdult.isTrue)
                )
            } else {
                where.and(
                    event.isAdult.isNull
                        .or(event.isAdult.isFalse)
                )
            }
        }

        return queryFactory
            .select(
                QEventItem(
                    event.id,
                    event.title,
                    event.thumbnailImage,
                    event.detailImage,
                    event.popupImage,
                    event.link,
                    event.isAdult,
                    event.isPopup
                )
            )
            .from(event)
            .where(where)
            .orderBy(event.id.desc())
            .fetch()
    }

    override fun getMainEventPopup(isAdult: Boolean): EventItem? {
        var where = event.isActive.isTrue
            .and(event.isPopup.isTrue)
            .and(event.popupImage.isNotNull)

        where = if (isAdult) {
            where.and(
                event.isAdult.isNull
                    .or(event.isAdult.isTrue)
            )
        } else {
            where.and(
                event.isAdult.isNull
                    .or(event.isAdult.isFalse)
            )
        }

        return queryFactory
            .select(
                QEventItem(
                    event.id,
                    event.title,
                    event.thumbnailImage,
                    event.detailImage,
                    event.popupImage,
                    event.link,
                    event.isAdult,
                    event.isPopup
                )
            )
            .from(event)
            .where(where)
            .orderBy(event.id.desc())
            .fetchFirst()
    }
}
