package kr.co.vividnext.sodalive.live.room.kickout

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.live.room.LiveRoomRepository
import kr.co.vividnext.sodalive.live.room.info.LiveRoomInfoRedisRepository
import kr.co.vividnext.sodalive.live.room.info.LiveRoomMember
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRepository
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service

@Service
class LiveRoomKickOutService(
    private val roomInfoRepository: LiveRoomInfoRedisRepository,
    private val repository: LiveRoomKickOutRedisRepository,
    private val memberRepository: MemberRepository,
    private val roomRepository: LiveRoomRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) {
    fun kickOut(request: LiveRoomKickOutRequest, member: Member) {
        val room = roomRepository.findByIdOrNull(request.roomId)
            ?: throw SodaException("해당하는 라이브가 없습니다.")

        val roomInfo = roomInfoRepository.findByIdOrNull(request.roomId)
            ?: throw SodaException("해당하는 라이브의 정보가 없습니다.")

        if (room.member == null || room.member!!.id == null) {
            throw SodaException("해당하는 라이브가 없습니다.")
        }

        if (!roomInfo.managerList.contains(LiveRoomMember(member, cloudFrontHost)) && room.member!!.id != member.id) {
            throw SodaException("권한이 없습니다.")
        }

        var liveRoomKickOut = repository.findByIdOrNull(request.roomId)
        if (liveRoomKickOut == null) {
            liveRoomKickOut = repository.save(LiveRoomKickOut(roomId = request.roomId))
        }

        liveRoomKickOut.kickOut(request.userId)
        repository.save(liveRoomKickOut)

        val kickOutUser = memberRepository.findByIdOrNull(request.userId)
        if (kickOutUser != null) {
            roomInfo.removeSpeaker(kickOutUser)
            roomInfo.removeListener(kickOutUser)
            roomInfo.removeManager(kickOutUser)
            roomInfoRepository.save(roomInfo)
        }
    }

    fun getKickOutCount(roomId: Long, userId: Long): Int {
        val liveRoomKickOut = repository.findByIdOrNull(roomId) ?: return 0

        val findUser = liveRoomKickOut.userList.find { it.userId == userId } ?: return 0
        return findUser.count
    }

    fun deleteKickOutData(roomId: Long) {
        repository.deleteById(roomId)
    }
}
