package kr.co.vividnext.sodalive.live.roulette

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRole
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/roulette")
class RouletteController(private val service: RouletteService) {
    @PostMapping
    fun createOrUpdateRoulette(
        @RequestBody request: CreateOrUpdateRouletteRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null || member.role != MemberRole.CREATOR) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        ApiResponse.ok(service.createOrUpdateRoulette(memberId = member.id!!, request = request))
    }

    @GetMapping
    fun getRoulette(
        @RequestParam creatorId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ): ApiResponse<GetRouletteResponse> {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        return ApiResponse.ok(service.getRoulette(creatorId = creatorId, memberId = member.id!!))
    }

    @PostMapping("/spin")
    fun spinRoulette(
        @RequestBody request: SpinRouletteRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.spinRoulette(request = request, memberId = member.id!!))
    }

    @PostMapping("/refund/{id}")
    fun refundDonation(
        @PathVariable id: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.refundDonation(id, member))
    }
}
